#!/usr/bin/env python
import json
import requests
import argparse


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("--notification_type", required=True,
                        help="Type of notification, host or service")
    parser.add_argument("--state", required=True,
                        help="State of host/service, OK, WARNING, \
                            CRITICAL, DOWN, UNREACHABLE")
    parser.add_argument("--object", required=True,
                        help="Name of object")
    parser.add_argument("--output", required=True,
                        help="Output to display")
    parser.add_argument("--webhook_url", required=True,
                        help="URL to your webhook in slack")
    parser.add_argument("--verbose", '-v', action='count',
                        help="Run tests")
    return parser.parse_args()


def state_to_color(state):
    """ Returns the color corresponding to the state
    If wrong argument is called, we return our default blue.

    >>> state_to_color("CUSTOM")
    '#0000FF'
    >>> state_to_color("ACKNOWLEDGEMENT")
    '#0000FF'
    >>> state_to_color("OK")
    'good'
    >>> state_to_color("DOWN")
    'danger'
    >>> state_to_color("untracked")
    '#0000FF'
    >>> state_to_color(1)
    '#0000FF'
    >>> state_to_color([])
    '#0000FF'
    >>> state_to_color({})
    '#0000FF'

    """

    state_to_color = {
        'CUSTOM': '#0000FF',
        'ACKNOWLEDGEMENT': '#0000FF',
        'OK': 'good',
        'RECOVERY': 'good',
        'WARNING': 'warning',
        'PROBLEM': 'warning',
        'UNREACHABLE': 'warning',
        'CRITICAL': 'danger',
        'DOWN': 'danger'
    }

    try:
        color = state_to_color[state]
    except KeyError:
        color = '#0000FF'
    except TypeError:
        color = '#0000FF'

    return color


def build_message(state, notification_type, object, output):
    """ Returns a formatted string
    "[OK] <notification_type:<object> output:<output>"
    >>> build_message("PROBLEM", "host", "monitor", "my output")
    '[PROBLEM] host: monitor output: my output'
    >>> build_message("OK", "service", "monitor;Disk Usage", "my output")
    '[OK] service: monitor;Disk Usage output: my output'

    """
    text = "[{}] {}: {} output: {}".format(state,
                                           notification_type,
                                           object,
                                           output)
    return text


def build_json_payload(text, color):
    """ Returns json formatted payload data
    >>> build_json_payload('My text', 'good')
    {'username': 'Octobot', 'attachments': [{'color': 'good', 'fields': \
[{'short': False, 'value': 'My text', 'title': 'Notification'}], \
'fallback': 'My text'}], 'icon_emoji': ':octopus:'}
    >>> build_json_payload('Other text', '#0000FF')
    {'username': 'Octobot', 'attachments': [{'color': '#0000FF', 'fields': \
[{'short': False, 'value': 'Other text', 'title': 'Notification'}], \
'fallback': 'Other text'}], 'icon_emoji': ':octopus:'}
    """
    slack_data = {
        'username': "Octobot",
        'icon_emoji': ":octopus:",
        "attachments": [{
            "fallback": text,
            "color": color,
            "fields": [{
                "title": "Notification",
                "value": text,
                "short": False
            }]
        }]
    }
    return slack_data


def send_request(webhook_url, slack_data):
    response = requests.post(
        webhook_url, data=json.dumps(slack_data),
        headers={'Content-Type': 'application/json'}
    )
    if response.status_code != 200:
        raise ValueError(
            'Request to slack returned an error {}, the response is:\n{}'
            .format(response.status_code, response.text)
        )


if __name__ == '__main__':
    import doctest
    doctest.testmod()
    args = parse_arguments()
    send_request(args.webhook_url, build_json_payload(
                                build_message(
                                    args.state,
                                    args.notification_type,
                                    args.object,
                                    args.output
                                ), state_to_color(args.state)))
